@@ -1,4 +1,4 @@
-#!python
+#!/usr/bin/env python
 
 # Setup file for pybench
 #
@@ -30,6 +30,10 @@ from Lists import *
 from Tuples import *
 from Dict import *
 from Exceptions import *
+try:
+    from With import *
+except SyntaxError:
+    pass
 from Imports import *
 from Strings import *
 from Numbers import *
